@@ -327,7 +327,7 @@ class PackageList:
                 if arch not in valid_architectures:
                     raise ValueError, "Invalid architecture name."
                 
-                raw = os.popen("cpp -I %s \
+                raw = os.popen("cpp -undef -I %s \
                                     -I /usr/share/debian-cd/tasks \
                                     -DARCH_%s %s | \
                                 grep -v \'^$\|^#.*$\' | tr \'\n\' \' \'"
@@ -402,10 +402,10 @@ def _virtual_as_alternative(original, full_lists):
                 found = False
                 for pkglist in full_lists:
                     if pkgname in pkglist._provides:
-                        # First, if there is a non-vitual package, add it
+                        # if there is a non-vitual package, add it
                         if pkgname in pkglist:
                             new_dep.append((pkgname, pkgver, cond))
-                        # And later add the packages that provide the package
+                        # add the packages that provide the package
                         for alternative in pkglist._provides[pkgname]:
                             new_dep.append((alternative['Package'], '', ''))
                         found = True
@@ -438,32 +438,36 @@ def resolve_depends(from_list, with_lists, field=DEPENDS):
                                     goodness[pkgname] = 0
                             break
 
-        if not depends:
-            # Packages without depends has score 1 (itself)
-            return 1
-        else:
-            # Packages with depends has score 1 plus the minimum score
-            # of all alternatives for its depends
-            score = 1
-            for depend in depends:
-                depend_score = 0
-                for pkgname, pkgver, cond in depend:
-                    if scores.has_key(pkgname):
-                        my_score = scores[pkgname]
-                        if my_score < depend_score or depend_score == 0:
-                            depend_score = my_score
-                score += depend_score
+        # Packages with depends has score 1 plus the minimum score
+        # of all alternatives for its depends
+        score = 1
+        for depend in depends:
+            depend_score = 0
+            for pkgname, pkgver, cond in depend:
+                if scores.has_key(pkgname):
+                    my_score = scores[pkgname]
+                    if my_score < depend_score or depend_score == 0:
+                        depend_score = my_score
+            score += depend_score
 
-            return score
+        return score
             
     def select_field(package, backtrace = []):
         # Select packages to minimize the total score
         depends = _virtual_as_alternative(package[field], with_lists)
+        selected = []
         for depend in depends:
             found = False
             depend_score = 0
             goodness_score = 0
             for pkgname, pkgver, cond in depend:
+                # If any alternative is already selected, use it
+                if pkgname in from_list:
+                    found = True
+                    selected.append(pkgname)
+                    break
+
+                # Choose the alternative with best goodness and smalest score 
                 if scores.has_key(pkgname):
                     found = True
                     
@@ -474,7 +478,7 @@ def resolve_depends(from_list, with_lists, field=DEPENDS):
                              my_goodness > goodness_score):
                         depend_score = my_score
                         goodness_score = my_goodness
-                        selected = pkgname
+                        selected.append(pkgname)
 
             if not found:
                 if depends:
@@ -483,13 +487,14 @@ def resolve_depends(from_list, with_lists, field=DEPENDS):
                         broken_depends[package["Package"]] = depends
                     return False
             
+        for pkg in selected:
             for pkglist in with_lists:
-                if selected in pkglist:
-                    if selected not in from_list:
-                        from_list.add(pkglist[selected], pkglist)
-                    if selected not in backtrace:
-                        backtrace.append(selected)
-                        select_field(pkglist[selected], backtrace)
+                if pkg in pkglist:
+                    if pkg not in from_list:
+                        from_list.add(pkglist[pkg], pkglist)
+                        backtrace.append(pkg)
+                        select_field(pkglist[pkg], backtrace)
+                        break
 
     # Convert with_list to a list
     if not isinstance(with_lists, list):
