@@ -176,6 +176,30 @@ class PackageListTests(TestBase):
         self.failUnlessEqual(len(pkg_list), 3,
                              "Selected too many packages")
 
+    def test_5depends_real_or_virtual (self):
+        """PackageList: A depends on B | virtualC, D provides virtualC"""
+        pkg_list = self.__create_list(self.aux_file('Packages-real_or_virtual'))
+
+        pkg_list_a_d = pkg_list.filter({'name':'a'})
+        pkg_list_a_d.add(pkg_list['d'])
+        pkg_list_a_d.resolve_depends(pkg_list)
+
+        # There is only one package in section graphics
+        self.failUnlessEqual(len(pkg_list_a_d), 2,
+                             "Selected too many packages")
+
+    def test_5depends_real_or_virtual2 (self):
+        """PackageList: META depends on A and D, A depends on B | virtualC, D provides virtualC"""
+        pkg_list = self.__create_list(self.aux_file('Packages-real_or_virtual'))
+
+        pkg_list_selected = pkg_list.filter({'name':'meta'})
+        pkg_list_selected.resolve_depends(pkg_list)
+
+        # There is only one package in section graphics
+        self.failUnlessEqual(len(pkg_list_selected), 3,
+                             "Selected too many packages: %s" % map(lambda x: x['Package'],
+                                                                    pkg_list_selected.values()))
+
     def test_6broken_dependencies (self):
         """PackageList: broken dependencies"""
         pkg_list = self.__create_list(self.aux_file('BrokenPackages-a-b--d'))
@@ -256,6 +280,14 @@ class PackageListTests(TestBase):
                             "Returned a invalid adress")
         
         
+    def test_13virtual_looping (self):
+        """PackageList: virtual package looping"""
+        pkg_list = self.__create_list(self.aux_file('VirtualPackagesLooping'))
+        try:
+            pkg_list.resolve_depends(pkg_list)
+        except RuntimeError, e:
+            self.fail("Raised a RuntimeError exception.", e)
+
 def suite():
     suite = unittest.TestSuite()
 
